@@ -23,7 +23,6 @@ def extractNSBMD(infolder, outfolder, extension=".nsbmd"):
 
 
 def repackNSBMD(workfolder, infolder, outfolder, extension=".nsbmd", writefunc=None):
-    common.makeFolder(outfolder)
     common.logMessage("Repacking NSBMD from", workfolder, "...")
     files = common.getFiles(infolder, extension)
     for file in common.showProgress(files):
@@ -114,76 +113,176 @@ def repackIMG(workfolder, infolder, outfolder, extensions=".NCGR", readfunc=None
             common.copyFile(infolder + mapfile, outfolder + mapfile)
             writeMappedNSCR(outfolder + file, outfolder + mapfile, image, map, workfolder + pngfile, palettes, width, height, transptile)
         else:
-            common.copyFile(infolder + cellfile, outfolder + cellfile)
+            if os.path.isfile(workfolder + cellfile):
+                cell = readNCER(workfolder + cellfile)
+                common.copyFile(workfolder + cellfile, outfolder + cellfile)
+            else:
+                common.copyFile(infolder + cellfile, outfolder + cellfile)
             writeNCER(outfolder + file, outfolder + cellfile, image, cell, workfolder + pngfile, palettes, width, height)
     common.logMessage("Done!")
 
 
 # Font
-def getFontGlyphs(file):
+class FontNFTR:
+    height = 0
+    width = 0
+    plgcoffset = 0
+    hdwcoffset = 0
+    pamcoffset = 0
+    plgcsize = 0
+    glyphwidth = 0
+    glyphheight = 0
+    glyphbytes = 0
+    pixelbits = 0
+    tilenum = 0
+    firstcode = 0
+    lastcode = 0
+    plgc = []
+    colors = []
+    hdwc = []
+    pamc = []
     glyphs = {}
+
+
+class FontHDWC:
+    start = 0
+    width = 0
+    length = 0
+
+
+class FontPAMC:
+    firstchar = 0
+    lastchar = 0
+    type = 0
+    nextoffset = 0
+
+
+class FontGlyph:
+    start = 0
+    width = 0
+    length = 0
+    char = ""
+    code = 0
+    index = 0
+
+    def __init__(self, hdwc, char, code, index):
+        self.start = hdwc.start
+        self.width = hdwc.width
+        self.length = hdwc.length
+        self.char = char
+        self.code = code
+        self.index = index
+
+
+def readNFTR(file):
+    nftr = FontNFTR()
     with common.Stream(file, "rb") as f:
         # Header
         f.seek(25)
-        fontheight = f.readByte()
+        nftr.height = f.readByte()
         f.seek(3, 1)
-        fontwidth = f.readByte()
+        nftr.width = f.readByte()
         f.seek(2, 1)
-        plgcoffset = f.readUInt()
-        hdwcoffset = f.readUInt()
-        pamcoffset = f.readUInt()
-        common.logDebug("fontwidth:", fontwidth, "fontheight:", fontheight, "plgcoffset:", plgcoffset, "hdwcoffset:", hdwcoffset, "pamcoffset:", pamcoffset)
+        nftr.plgcoffset = f.readUInt()
+        nftr.hdwcoffset = f.readUInt()
+        nftr.pamcoffset = f.readUInt()
         # PLGC
-        f.seek(plgcoffset - 4)
-        plgcsize = f.readUInt()
+        f.seek(nftr.plgcoffset - 4)
+        nftr.plgcsize = f.readUInt()
+        nftr.glyphwidth = f.readByte()
+        nftr.glyphheight = f.readByte()
+        nftr.glyphbytes = f.readUShort()
         f.seek(2, 1)
-        tilelength = f.readUShort()
-        tilenum = (plgcsize - 0x10) // tilelength
-        common.logDebug("plgcsize:", plgcsize, "tilelength:", tilelength, "tilenum:", tilenum)
+        nftr.pixelbits = f.readUShort()
+        nftr.tilenum = (nftr.plgcsize - 0x10) // nftr.glyphbytes
+        common.logDebug(vars(nftr))
+        # Generate colors
+        nftr.colors = []
+        numcolors = pow(2, nftr.pixelbits)
+        for i in range(numcolors):
+            nftr.colors.append((0, 0, 0, int(255 * i / (numcolors - 1))))
+        # Read the glyphs graphics
+        nftr.plgc = []
+        data = f.readByte()
+        for i in range(nftr.tilenum):
+            glyph = Image.new("RGBA", (nftr.glyphwidth, nftr.glyphheight), nftr.colors[0])
+            pixels = glyph.load()
+            x = 0
+            y = 0
+            byteindex = 0
+            bitmask = 0x80
+            while byteindex < nftr.glyphbytes and y < nftr.glyphheight:
+                intensitymask = pow(2, nftr.pixelbits - 1)
+                intensity = 0
+                while intensitymask > 0:
+                    if data & bitmask > 0:
+                        intensity += intensitymask
+                    bitmask >>= 1
+                    if bitmask == 0:
+                        bitmask = 0x80
+                        byteindex += 1
+                        data = f.readByte()
+                    intensitymask >>= 1
+                if intensity > 0:
+                    pixels[x, y] = nftr.colors[intensity]
+                x += 1
+                if x >= nftr.glyphwidth:
+                    x = 0
+                    y += 1
+            while byteindex < nftr.glyphbytes:
+                data = f.readByte()
+                byteindex += 1
+            nftr.plgc.append(glyph)
         # HDWC
-        f.seek(hdwcoffset)
-        firstcode = f.readUShort()
-        lastcode = f.readUShort()
+        f.seek(nftr.hdwcoffset)
+        nftr.firstcode = f.readUShort()
+        nftr.lastcode = f.readUShort()
         f.seek(4, 1)
-        common.logDebug("firstcode:", firstcode, "lastcode:", lastcode)
-        hdwc = []
-        for i in range(tilenum):
-            hdwcstart = f.readSByte()
-            hdwcwidth = f.readByte()
-            hdwclength = f.readByte()
-            hdwc.append((hdwcstart, hdwcwidth, hdwclength))
+        nftr.hdwc = []
+        for i in range(nftr.tilenum):
+            hdwc = FontHDWC()
+            hdwc.start = f.readSByte()
+            hdwc.width = f.readByte()
+            hdwc.length = f.readByte()
+            common.logDebug(" ", vars(hdwc))
+            nftr.hdwc.append(hdwc)
         # PAMC
-        nextoffset = pamcoffset
+        nftr.pamc = []
+        nftr.glyphs = {}
+        nextoffset = nftr.pamcoffset
         while nextoffset != 0x00:
             f.seek(nextoffset)
-            firstchar = f.readUShort()
-            lastchar = f.readUShort()
-            sectiontype = f.readUInt()
-            nextoffset = f.readUInt()
-            common.logDebug("firstchar:", common.toHex(firstchar), "lastchar:", common.toHex(lastchar), "sectiontype:", sectiontype, "nextoffset:", nextoffset)
-            if sectiontype == 0:
+            pamc = FontPAMC()
+            pamc.firstchar = f.readUShort()
+            pamc.lastchar = f.readUShort()
+            pamc.type = f.readUInt()
+            nextoffset = pamc.nextoffset = f.readUInt()
+            common.logDebug(" ", vars(pamc))
+            if pamc.type == 0:
                 firstcode = f.readUShort()
-                for i in range(lastchar - firstchar + 1):
-                    c = common.codeToChar(firstchar + i)
-                    glyphs[c] = hdwc[firstcode + i] + (firstchar + i,)
-            elif sectiontype == 1:
-                for i in range(lastchar - firstchar + 1):
+                for i in range(pamc.lastchar - pamc.firstchar + 1):
+                    c = common.codeToChar(pamc.firstchar + i)
+                    nftr.glyphs[c] = FontGlyph(nftr.hdwc[firstcode + i], c, pamc.firstchar + i, firstcode + i)
+            elif pamc.type == 1:
+                for i in range(pamc.lastchar - pamc.firstchar + 1):
                     charcode = f.readUShort()
-                    if charcode == 0xFFFF or charcode >= len(hdwc):
+                    if charcode == 0xFFFF or charcode >= len(nftr.hdwc):
                         continue
-                    c = common.codeToChar(firstchar + i)
-                    glyphs[c] = hdwc[charcode] + (firstchar + i,)
+                    c = common.codeToChar(pamc.firstchar + i)
+                    nftr.glyphs[c] = FontGlyph(nftr.hdwc[charcode], c, pamc.firstchar + i, charcode)
             else:
-                common.logError("Unknown section type", sectiontype)
-    return glyphs
+                common.logError("Unknown section type", pamc.type)
+    return nftr
 
 
 def extractFontData(fontfiles, out):
+    if isinstance(fontfiles, str):
+        fontfiles = [fontfiles]
     with common.Stream(out, "wb") as f:
         for fontfile in fontfiles:
-            glyphs = getFontGlyphs(fontfile)
+            nftr = readNFTR(fontfile)
             for i in range(0x20, 0x7e):
-                f.writeByte(glyphs[chr(i)][2])
+                f.writeByte(nftr.glyphs[chr(i)][2])
 
 
 # Graphics
@@ -1225,7 +1324,13 @@ def writeNSBMD(file, nsbmd, texi, infile, fixtrasp=False):
             common.logError("Texture format 1 not implemented")
         # 4-color Palette
         elif tex.format == 2:
-            common.logError("Texture format 2 not implemented")
+            for i in range(tex.height):
+                for j in range(0, tex.width, 4):
+                    index1 = common.getPaletteIndex(paldata, pixels[j, i], fixtrasp)
+                    index2 = common.getPaletteIndex(paldata, pixels[j + 1, i], fixtrasp)
+                    index3 = common.getPaletteIndex(paldata, pixels[j + 2, i], fixtrasp)
+                    index4 = common.getPaletteIndex(paldata, pixels[j + 3, i], fixtrasp)
+                    f.writeByte((index4 << 6) | (index3 << 4) | (index2 << 2) | index1)
         # 16/256-color Palette
         elif tex.format == 3 or tex.format == 4:
             for i in range(tex.height):
